//##################################################################################################
//
//  Ceetron Export
//  VTFxAppend
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################


#include "CeeVTFx/Database.h"
#include "CeeVTFx/GeometryBlock.h"
#include "CeeVTFx/File.h"
#include "CeeVTFx/StateInfoBlock.h"
#include "CeeVTFx/GeometryInfoBlock.h"
#include "CeeVTFx/ResultBlock.h"
#include "CeeVTFx/NodeBlock.h"
#include "CeeVTFx/ElementBlock.h"
#include "CeeVTFx/ResultValuesBlock.h"
#include "CeeVTFx/VTFxComponent.h"

#include "CeeCore/CoreComponent.h"
#include "CeeCore/LogDestinationConsole.h"
#include "CeeCore/LogManager.h"
#include "CeeCore/Str.h"
#include "CeeCore/PropertySetCollection.h"
#include "CeeCore/PropertySet.h"

#include <iostream>
#include <vector>
#include <cstdlib>
#include <fstream>

//--------------------------------------------------------------------------------------------------
// Creates case properties
//--------------------------------------------------------------------------------------------------
bool createProperties(cee::vtfx::Case* currentCase)
{
    cee::PtrRef<cee::PropertySetCollection> vtfxProps = new cee::PropertySetCollection;

    // Set part (id = 1) to use line as draw style
    cee::PtrRef<cee::PropertySet> partSettings = new cee::PropertySet("part_settings");
    partSettings->setValue("context_geometry_index", static_cast<unsigned int>(0));
    partSettings->setValue("context_part_id", 1);
    partSettings->setValue("draw_style", "line");
    vtfxProps->addPropertySet(partSettings.get());

    // Set part (id = 2) to use red color and opacity
    cee::PtrRef<cee::PropertySet> partSettings2 = new cee::PropertySet("part_settings");
    partSettings2->setValue("context_geometry_index", static_cast<unsigned int>(0));
    partSettings2->setValue("context_part_id", 2);
    partSettings2->setValue("color", cee::Color3f(1.0f, 0.0f, 0.0f));
    partSettings2->setValue("opacity", 0.3f);
    vtfxProps->addPropertySet(partSettings2.get());

    // Set the properies
    return currentCase->setProperties(vtfxProps.get());
}


cee::PtrRef<cee::Instance>  g_componentInstance;

//--------------------------------------------------------------------------------------------------
// Main function, program entry point
//--------------------------------------------------------------------------------------------------
int main()
{
    // Initialize components and set up logging to console
    // -------------------------------------------------------------------------
    unsigned int licenseKeyA = 0x00;    // Todo: Set by developer 
    unsigned int licenseKeyB = 0x00;    // Todo: Set by developer
    g_componentInstance = cee::CoreComponent::initialize(licenseKeyA, licenseKeyB);
    cee::vtfx::VTFxComponent::initialize(g_componentInstance.get());

    cee::PtrRef<cee::LogDestinationConsole> log = new cee::LogDestinationConsole();
    cee::CoreComponent::logManager()->setDestination(log.get());
    cee::CoreComponent::logManager()->setLevel("", 3);               // Log errors, warnings and info

    // Create the VTFx file
    // -------------------------------------------------------------------------
    cee::Str fileName = "ExampleAppend.vtfx";

    // Make copy original file
    std::ifstream src("../../DemoFiles/SimpleExample.vtfx", std::ios::binary);
    if (!src.good())
    {
        return EXIT_FAILURE;
    }
    std::ofstream dst(fileName.toStdString().c_str(), std::ios::binary);
    dst << src.rdbuf();
    src.close();
    dst.close();

    cee::PtrRef<cee::vtfx::File> file = new cee::vtfx::File;
    if (!file->openForAppend(fileName))
    {
        return EXIT_FAILURE;
    }

    // Get the first database from file
    cee::PtrRef<cee::vtfx::Database> db = file->database(0); 
    
    // Create a single case (could have many)
    // -------------------------------------------------------------------------
    cee::PtrRef<cee::vtfx::Case> newCase = new cee::vtfx::Case(file.get(), "Appended Case", file->unusedFileCaseId(), db->id()); 
    if (!createProperties(newCase.get()))
    {
        return EXIT_FAILURE;
    }

    // Finally close the file
    // -------------------------------------------------------------------------
    if (!file->close())
    {
        return EXIT_FAILURE;
    }
    
    std::cout << "Exported successfully to file: " << fileName.toStdString() << std::endl;
    std::cout << std::endl << "Press enter to exit..." << std::endl;
    std::cin.ignore();

    return EXIT_SUCCESS;
}
