//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"
#include "CeeCore/Str.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/PropertySetCollection.h"
#include "CeeCore/Image.h"

namespace cee {
namespace vtfx {

class File;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VTFX_EXPORT Case : public RefCountedObject
{
public:
    Case(File* file, const Str& name, int caseId, int databaseId);
    virtual ~Case();

    int                             id() const;
    int                             databaseId() const;
    cee::Str                        name() const;
    void                            setName(const cee::Str& name);
    Str                             description() const;
    void                            setDescription(const Str& descriptionHtml);
    const Image*                    snapshot() const;
    void                            setSnapshot(cee::Image* image);

    size_t                          imageCount() const;
    cee::Str                        imageName(size_t imageIndex) const;
    const Image*                    image(size_t imageIndex) const;
    bool                            addImage(const cee::Str& imageName, cee::Image* image);

    const PropertySetCollection*    properties() const;
    bool                            setProperties(PropertySetCollection* propertySetCollection);

    bool                            derivedResultsDisabled() const;
    void                            setDerivedResultsDisabled(bool disable);

private:
    Case(File* file, int caseId);

private:
    friend class FilePrivate;
    friend class File;
    CEE_PRIVATE_IMPL(Case);
    CEE_DISALLOW_COPY_AND_ASSIGN(Case);
};

}
}
