//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"
#include "CeeVTFx/Block.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Str.h"
#include "CeeCore/PtrRef.h"

namespace cee {
namespace vtfx {

class File;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VTFX_EXPORT Database : public RefCountedObject
{
public:
    // Simulation and solution types for SimulationInfoBlock
    // ----------------------------------------------------
    enum SimulationType
    {
        SIMULATION_TYPE_STRUCTURAL = 0,
        SIMULATION_TYPE_THERMAL = 1,
        SIMULATION_TYPE_ELECTRIC = 2,
        SIMULATION_TYPE_MAGNETIC = 3,
        SIMULATION_TYPE_FLUID = 4,
        SIMULATION_TYPE_ACOUSTIC = 5,
        SIMULATION_TYPE_DIFFUSION = 6,
        SIMULATION_TYPE_OTHER = 7
    };


    enum SolutionType
    {
        SOLUTION_TYPE_STATIC = 0,
        SOLUTION_TYPE_STEADY_STATE = 1,
        SOLUTION_TYPE_VIBRATION = 2,
        SOLUTION_TYPE_BUCKLING = 3,
        SOLUTION_TYPE_TRANSIENT = 4,
        SOLUTION_TYPE_SUPERELEMENT = 5,
        SOLUTION_TYPE_FREQRESPONSE = 6,
        SOLUTION_TYPE_COMPLEXEIGEN = 7,
        SOLUTION_TYPE_QUASISTATIC = 8,
        SOLUTION_TYPE_OUTPUT = 9,
        SOLUTION_TYPE_SINGULAR = 10,
        SOLUTION_TYPE_OTHER = 11
    };
    
public:
    Database(File* file, const Str& databaseName, int databaseId, SimulationType simulationType = SIMULATION_TYPE_OTHER, SolutionType solutionType = SOLUTION_TYPE_OTHER, const Str& description = "");
    virtual ~Database();

    int             id() const;
    Str             name() const;

    bool            writeBlock(const Block* block);

    size_t          blockCount(Block::BlockType blockType) const;
    PtrRef<Block>   blockByIndex(Block::BlockType blockType, size_t blockIndex);
    PtrRef<Block>   blockById(Block::BlockType blockType, int blockId);

private:
    Database(File* file, int databaseId); 
    bool            internal_parse();

private:
    friend class FilePrivate; 
    friend class File; 
    
    CEE_PRIVATE_IMPL(Database);
    CEE_DISALLOW_COPY_AND_ASSIGN(Database);
};

}
}
