//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"
#include "CeeVTFx/Block.h"

#include <vector>

class VTFxElementBlock;

namespace cee {
namespace vtfx {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VTFX_EXPORT ElementBlock : public Block
{
public:
    // Element types for ElementBlock
    // -----------------------------------------------------------------------------
    enum ElementType {
        POINTS,
        BEAMS,
        BEAMS_3,
        TRIANGLES,
        TRIANGLES_6,
        QUADS,
        QUADS_8,
        QUADS_9,
        PYRAMIDS,
        PYRAMIDS_13,
        TETRAHEDRONS,
        TETRAHEDRONS_10,
        HEXAHEDRONS,
        HEXAHEDRONS_20,
        PENTAHEDRONS,
        PENTAHEDRONS_15,
        POLYHEDRONS,
        UNKNOWN_ELEMENT
    };


public:
    ElementBlock(int blockId, bool withElementIds, bool referNodesByIds);
    virtual ~ElementBlock();

    size_t          elementCount() const;
    size_t          elementNodeCount() const;
    bool            hasElementIds() const;
    bool            referNodesByIds() const;

    size_t          elementGroupCount() const;
    ElementType     elementGroupType(size_t groupIndex) const;
    size_t          elementGroupElementCount(size_t groupIndex) const;
    size_t          elementGroupElementNodeCount(size_t groupIndex) const;
    bool            elementGroup(size_t groupIndex, std::vector<int>* elementNodes, std::vector<int>* elementIds) const;
    bool            elementIndices(size_t groupIndex, std::vector<int>* elementIndices) const;
    
    bool            polyhedronsFaceCounts(size_t groupIndex, std::vector<int>* faceCounts);
    bool            polyhedronsFaceNodeCounts(size_t groupIndex, std::vector<int>* faceNodeCounts);
    bool            polyhedronsLocalNodes(size_t groupIndex, std::vector<int>* localNodes);
    bool            polyhedronsElementNodeCounts(size_t groupIndex, std::vector<int>* elementNodeCounts);

    void            setElements(const std::vector<ElementType>& elementTypes, const std::vector<int>& elementNodes, const std::vector<int>& elementIds = std::vector<int>());

    bool            addElements(ElementType elementType, const std::vector<int>& elementNodes, const std::vector<int>& elementIds = std::vector<int>());
    bool            addElements(ElementType elementType, const int* elementNodes, size_t elementNodeCount, const int* elementIds = NULL);

    bool            addElement(ElementType elementType, const std::vector<int>& elementNodes, int elementId = -1);
    bool            addElement(ElementType elementType, const int* elementNodes, size_t count, int elementId = -1);

    int             nodeBlockId() const;
    void            setNodeBlockId(int nodeBlockId);

    bool            checkValidity() const;

    // Static helper functions for the element library
    static size_t                       nodesPerElement(ElementType elementType);
    static bool                         isVolumeElement(ElementType elementType);

    static size_t                       surfacesPerElement(ElementType elementType);
    static std::vector<unsigned int>    elementSurface(ElementType elementType, size_t surfaceIndex);

private:
    ElementBlock(VTFxElementBlock* block, int blockId);

    VTFxBlock*  internal_vtfxBlock() const;

private:
    friend class DatabasePrivate;

    CEE_PRIVATE_IMPL(ElementBlock);
    CEE_DISALLOW_COPY_AND_ASSIGN(ElementBlock);
};

}
}
