//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"
#include "CeeVTFx/Block.h"
#include "CeeVTFx/FileSettings.h"
#include "CeeVTFx/Case.h"

#include "CeeCore/PtrRef.h"


class VTFxAccess;

namespace cee {
namespace vtfx {

class Database;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VTFX_EXPORT File : public RefCountedObject
{
public:
    File();
    virtual ~File();

    bool                    open(const Str& filename);
    bool                    create(const Str& filename, const FileSettings& fileSettings);
    bool                    openForAppend(const Str& filename); 
    bool                    close();

    size_t                  userTextFileCount() const;
    Str                     userTextFileName(size_t textFileIndex) const;
    Str                     userTextFileContent(size_t textFileIndex) const;
    bool                    addUserTextFile(const Str& filename, const Str& content);

    size_t                  databaseCount() const;
    int                     unusedDatabaseId() const;
    Database*               database(size_t databaseIndex);

    size_t                  fileCaseCount() const; 
    int                     unusedFileCaseId() const; 
    const Case*             fileCase(size_t caseIndex) const;
    Case*                   fileCase(size_t caseIndex);

    const FileSettings*     settings() const;
    
    static bool             isVTFxFile(const Str& filename);

public:
    VTFxAccess*             internal_access();

private:
    friend class Database;
    friend class Case;    

    CEE_PRIVATE_IMPL(File);
    CEE_DISALLOW_COPY_AND_ASSIGN(File);
};

}
}
