//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"

#include "CeeCore/Str.h"


namespace cee {
namespace vtfx {

//=================================================================================================================================
//
/// \class cee::vtfx::FileSettings
/// \ingroup VTFx
/// 
/// %File settings for VTFx files. 
/// 
/// %File settings are provided when creating VTFx files and when appending data to VTFx files.
//
//=================================================================================================================================
class CEE_VTFX_EXPORT FileSettings
{
public:
    FileSettings() 
    {
        binary                  = true;
        createDigitalSignature  = false;
        compressionLevel        = 5;
        applicationName         = "";
        vendorName              = "";
        password                = "";
    }

    bool binary;                 ///< true (default): use binary data files <br />false: use text data files (mainly for debugging)
    bool createDigitalSignature; ///< true: a signature on the file contents will be created to detect changes made by third parties  <br />false (default): no signature will be created
    int compressionLevel;        ///< 0: no compression <br />1: low compression rate with fast decompression speed  <br />... <br />5 (default): medium compression <br />... <br />9: maximum compression
    Str vendorName;              ///< Vendor name, usually the company name (single line of text)
    Str applicationName;         ///< Name of the application that exports the VTFx file (single line of text)
    Str password;                ///< Password for encrypted VTFx files (AES 256)  <br />Empty string (default) to disable encryption.<br />Password must consist of 7 bit ASCII characters
};

}
}
