//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"
#include "CeeVTFx/Block.h"

#include "CeeCore/Vec3f.h"

#include <vector>

class VTFxNodeBlock;

namespace cee {
namespace vtfx {

//=================================================================================================================================
//
// 
//
//=================================================================================================================================
class CEE_VTFX_EXPORT NodeBlock : public Block
{
public:
    NodeBlock(int blockId, bool withNodeIds);
    virtual ~NodeBlock();

    size_t              nodeCount() const;
    std::vector<int>    nodeIds() const;

    std::vector<Vec3f>  nodes() const;
    bool                setNodes(const std::vector<float>& coordinates, const std::vector<int>& nodeIds = std::vector<int>());
    bool                setNodes(const std::vector<Vec3f>& coordinates, const std::vector<int>& nodeIds = std::vector<int>());

    bool                allocateNodes(size_t numNodes); 
    bool                addNode(float x, float y, float z, int nodeId = -1);
    bool                addNode(Vec3f node, int nodeId = -1);
    
    int                 nodeIndex(int nodeId);

private:
    NodeBlock(VTFxNodeBlock* block, int blockId);

    VTFxBlock*          internal_vtfxBlock() const;

private:
    friend class DatabasePrivate;

    CEE_PRIVATE_IMPL(NodeBlock);
    CEE_DISALLOW_COPY_AND_ASSIGN(NodeBlock);
};

}
}
