//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"
#include "CeeVTFx/Block.h"
#include "CeeCore/Str.h"

class VTFxResultBlock;

namespace cee {
namespace vtfx {

//=================================================================================================================================
//
//
//
//=================================================================================================================================
class CEE_VTFX_EXPORT ResultBlock : public Block
{
public:
    // Result types for ResultBlock
    // -----------------------------------------------------------------------------
    enum ResultType {
        UNKNOWN_TYPE = -1,
        SCALAR = 0,
        VECTOR = 1,
        DISPLACEMENT = 2,
        TENSOR = 3
    };

    // Result mapping types for ResultBlock
    // -----------------------------------------------------------------------------
    enum ResultMapping {
        UNKNOWN_MAPPING = -1,
        NODE_MAPPING = 0,
        ELEMENT_MAPPING = 1,
        ELEMENT_NODE_MAPPING = 3,
        ELEMENT_FACE_MAPPING = 4
    };


    // Strain types for ResultBlock
    // -----------------------------------------------------------------------------
    enum StrainType 
    {
        STRAINTYPE_NONE = -1,
        STRAINTYPE_ENGINEERING = 0,
        STRAINTYPE_STANDARD = 1
    };

public:
    ResultBlock(int blockId, ResultType resultType, ResultMapping resultMapping);
    ~ResultBlock();

    Str                 name() const;
    void                setName(const Str& resultName);

    int                 resultId() const;
    void                setResultId(int resultId);

    std::vector<int>    resultValuesBlocks(size_t stateIndex) const;
    bool                setResultValuesBlocks(const std::vector<int>& resultValuesBlockIds, int stateId = 1);
    bool                addResultValuesBlock(int resultValuesBlockId, int stateId = 1);

    bool                relativeDisplacementResults() const;
    void                setRelativeDisplacementResults(bool relative);

	void				setDefaultDisplacementScaleFactor(float scale);
	float				defaultDisplacementScaleFactor() const;

    StrainType          strainType() const;
    void                setStrainType(StrainType type);

    void                addAttribute(const Str& attributeName, const Str& attributeValue);
    Str                 attribute(const Str& attributeName) const;
    std::vector<Str>    attributeNames() const;

    ResultType          resultType() const;
    ResultMapping       resultMapping() const;
    
    size_t              stateCount() const;
    size_t              stateIndex(int stateId) const;
    int                 stateId(size_t stateIndex) const;

    bool                checkValidity() const;

private:
    ResultBlock(VTFxResultBlock* block, int blockId);

    VTFxBlock*          internal_vtfxBlock() const;

private:
    friend class DatabasePrivate;

    CEE_PRIVATE_IMPL(ResultBlock);
    CEE_DISALLOW_COPY_AND_ASSIGN(ResultBlock);

};

}
}
