//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"
#include "CeeVTFx/Block.h"

#include "CeeCore/Vec3f.h"

#include <vector>

class VTFxResultValuesBlock;

namespace cee {
namespace vtfx {

//=================================================================================================================================
//
//
//
//=================================================================================================================================
class CEE_VTFX_EXPORT ResultValuesBlock : public Block
{
public:
    // Result value dimensions for ResultValuesBlock
    // -----------------------------------------------------------------------------
    enum ResultValueDimension {
        SCALAR1D = 1,
        VECTOR3D = 3,
        TENSOR6D = 6
    };

public:
    ResultValuesBlock(int blockId, ResultValueDimension dimension, bool mapToItemIds);
    ~ResultValuesBlock();

    int                     mapToBlockId() const; 
    bool                    setMapToBlockId(int blockId, BlockType blockType);
    std::vector<int>        mapToItemIds() const; 
    bool                    hasMapToItemIds() const;
    ResultValueDimension    dimension() const;

    std::vector<float>      resultValues() const;
    bool                    setResultValues(const std::vector<float>& resultValues, const std::vector<int>& mapToItemIds = std::vector<int>());

    bool                    allocateResultValues(int numResultValues);
    bool                    addResultValue1d(float scalarResultValue, int mapToItemId = -1);
    bool                    addResultValue3d(float vectorResultValueX, float vectorResultValueY, float vectorResultValueZ, int mapToItemId = -1);

    bool                    checkValidity() const;

private:
    ResultValuesBlock(VTFxResultValuesBlock* block, int blockId);

    VTFxBlock*              internal_vtfxBlock() const;

private:
    friend class DatabasePrivate;

    CEE_PRIVATE_IMPL(ResultValuesBlock);
    CEE_DISALLOW_COPY_AND_ASSIGN(ResultValuesBlock);
};

}
}
