//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Str.h"
#include "CeeVTFx/Base.h"
#include "CeeVTFx/Block.h"


class VTFxSetBlock;
struct VTFxSetBlockItems;

namespace cee {
namespace vtfx {
    
//=================================================================================================================================
//
// 
//
//=================================================================================================================================
class CEE_VTFX_EXPORT SetItemGroup : public RefCountedObject
{
public:
    SetItemGroup();
    virtual ~SetItemGroup();

    int     blockId() const;
    void    setBlockId(int blockId);

    Block::BlockType    blockType() const;
    void                setBlockType(Block::BlockType blockType);

    size_t  itemCount() const;
    int     itemIdOrIndex(size_t index) const;

    void    setItems(const std::vector<int>& itemIdOrIndices);

    void    addItems(const std::vector<int>& itemIdOrIndices);
    void    addItem(int itemIdOrIndex);

    VTFxSetBlockItems*  internal_vtfxSetBlockItems();

private:
    CEE_PRIVATE_IMPL(SetItemGroup);
    CEE_DISALLOW_COPY_AND_ASSIGN(SetItemGroup);
};


//=================================================================================================================================
//
// 
//
//=================================================================================================================================
class CEE_VTFX_EXPORT SetBlock : public Block
{
public:
    /// Item types for SetBlock
    enum ItemType {
        ELEMENT,            ///< Element item type
        UNKNOWN_ITEM_TYPE   ///< Unknown item type
    };

public:
    SetBlock(int blockId, bool itemsByIds);
    virtual ~SetBlock();

    int		    setId() const;
    void		setSetId(int setId);

    Str	        setName() const;
    void		setSetName(const Str& name);
    
    ItemType	itemType() const;
    void		setItemType(ItemType itemType);
    
    size_t      totalItemCount() const;

    int			                itemGroupCount() const;
    SetItemGroup*		        itemGroup(size_t index);
    const SetItemGroup*	        itemGroup(size_t index) const;

    void						addItemGroup(SetItemGroup* itemGroup);
    void                        addItem(int itemIdOrIndex, int itemGroupId);
    void                        addItems(const int* itemIndicesOrIds, size_t itemCount, int itemGroupId);

private:
    SetBlock(VTFxSetBlock* block, int blockId);

    VTFxBlock*          internal_vtfxBlock() const;

private:
    friend class DatabasePrivate;

    CEE_PRIVATE_IMPL(SetBlock);
    CEE_DISALLOW_COPY_AND_ASSIGN(SetBlock);
};

}
}
