//##################################################################################################
//
//  Ceetron Export
//  Component: VTFx
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVTFx/Base.h"
#include "CeeCore/Str.h"

#include "CeeVTFx/Block.h"

class VTFxStateInfoBlock;

namespace cee {
namespace vtfx {

//=================================================================================================================================
//
//
//
//=================================================================================================================================
class CEE_VTFX_EXPORT StateInfoBlock : public Block
{
public:
    // State reference value types for StateInfoBlock
    // -----------------------------------------------------------------------------
    enum ReferenceValueType {
        TIME = 0,
        FREQUENCY = 1,
        LOADCASE = 2,
        OTHER = 3
    };

public:
    StateInfoBlock();
    ~StateInfoBlock();

    bool                        addStateInfo(int stateId, const Str& stateName, float stateReferenceValue, ReferenceValueType referenceValueType);
    
    size_t                      stateCount() const;
    size_t                      stateIndex(int stateId) const;

    int                         stateId(size_t stateIndex) const;
    Str                         stateName(size_t stateIndex) const;
    float                       stateReferenceValue(size_t stateIndex) const;
    ReferenceValueType          stateReferenceValueType(size_t stateIndex) const;
    
    bool                        addGroupInfo(int groupId, const Str& groupName, const std::vector<int>& stateIds);
    size_t                      groupCount() const;
    int                         groupId(size_t groupIndex) const;
    Str                         groupName(size_t groupIndex) const;
    std::vector<int>            groupStateIds(size_t groupIndex) const;

    bool                        checkValidity() const;

private:
    StateInfoBlock(VTFxStateInfoBlock* block, int blockId);

    VTFxBlock*                  internal_vtfxBlock() const;

private:
    friend class DatabasePrivate;

    CEE_PRIVATE_IMPL(StateInfoBlock);
    CEE_DISALLOW_COPY_AND_ASSIGN(StateInfoBlock);
};

}
}
