import sys
import os

# Check that the currently running Python interpreter has the same maj.min version 
# as the python version that the CPM was built against
if sys.version_info.major != 3 or sys.version_info.minor != 6:
    sys.stderr.write( "\n\n")
    sys.stderr.write("---------------------------------------------------------------------\n")
    sys.stderr.write("\n")
    sys.stderr.write("WARNING!!!!\n")
    sys.stderr.write("\n")
    sys.stderr.write("This version of Ceetron Python Modules was built for a different\n")
    sys.stderr.write("version of Python and will probably be incompatible.\n")
    sys.stderr.write("  CPM was built for Python version:   3.6\n")
    sys.stderr.write("  Current Python interpreter version: " + str(sys.version_info.major) + "." + str(sys.version_info.minor) + "\n")
    sys.stderr.write("\n")
    sys.stderr.write("---------------------------------------------------------------------\n")
    sys.stderr.write( "\n\n")


# For ordinary distributions of CPM it is assumed that all the native DLLs 
# reside in the same folder as this __init__.py ('cee') and we're basically good.
# However, we also allow for a slightly different 'dev' folder layout where the
# native DLLs are instead located one directory level above this folder. 
# For Python 3.8 this 'dev' layout needs special handling which is triggered
# by the presence of the cee_dev.py file
if os.name == "nt" and hasattr(os, "add_dll_directory"):
    try:
        # If we find the cee_dev.py file, assume we have 'dev' layout
        from . import cee_dev
        this_dir = os.path.dirname(os.path.abspath(__file__))
        os.add_dll_directory(os.path.abspath(os.path.join(this_dir, "..")))
    except:
        pass
